/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2021, Digium, Inc.
 *
 * Naveen Albert <asterisk@phreaknet.org>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Maximum number
 *
 * \author Naveen Albert <asterisk@phreaknet.org>
 *
 * \ingroup functions
 */

/*** MODULEINFO
	<support_level>core</support_level>
 ***/

#include "asterisk.h"

#include "asterisk/module.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/utils.h"
#include "asterisk/app.h"

/*** DOCUMENTATION
	<function name="MAX" language="en_US">
		<synopsis>
			Returns the maximum of two integers.
		</synopsis>
		<syntax>
			<parameter name="num1" />
			<parameter name="num2" />
		</syntax>
		<description>
			<para>Returns the maximum of two integers <replaceable>num1</replaceable> and <replaceable>num2</replaceable>.</para>
			<para>Example:  Set(max=${MAX(4,7)});
			Sets the max variable equal to 7.</para>
		</description>
	</function>
 ***/
static int acf_max_exec(struct ast_channel *chan, const char *cmd,
			 char *parse, char *buffer, size_t buflen)
{
	int int1, response_int, int2;
	AST_DECLARE_APP_ARGS(args,
			     AST_APP_ARG(num1);
			     AST_APP_ARG(num2);
	);

	AST_STANDARD_APP_ARGS(args, parse);

	if (ast_strlen_zero(args.num1) || sscanf(args.num1, "%30d", &int1) != 1)
		int1 = 0;

	if (ast_strlen_zero(args.num2) || sscanf(args.num2, "%30d", &int2) != 1)
		int2 = int1;
	
	response_int = (int1 < int2) ? int2 : int1;
	ast_debug(1, "%d is the maximum of [%d,%d]\n", response_int, int1, int2);
	snprintf(buffer, buflen, "%d", response_int);

	return 0;
}

static struct ast_custom_function acf_max = {
	.name = "MAX",
	.read = acf_max_exec,
	.read_max = 12,
};

static int unload_module(void)
{
	ast_custom_function_unregister(&acf_max);

	return 0;
}

static int load_module(void)
{
	return ast_custom_function_register(&acf_max);
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Maximum number dialplan function");